-- Copyright (C) 2023:
--   Helmholtz-Zentrum Potsdam Deutsches GeoForschungsZentrum GFZ
--
-- This program is free software: you can redistribute it and/or modify it
-- under the terms of the GNU Affero General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
-- General Public License for more details.
--
-- You should have received a copy of the GNU Affero General Public License
-- along with this program. If not, see http://www.gnu.org/licenses/.

-- Converts XY coordinates to a Quadkey.
-- Adapted from https://learn.microsoft.com/en-us/bingmaps/articles/bing-maps-tile-system

CREATE OR REPLACE FUNCTION tile_xy_to_quadkey(
    IN tile_x INT,
    IN tile_y INT,
    IN level_of_detail INT
)
RETURNS TEXT
LANGUAGE plpgsql IMMUTABLE PARALLEL SAFE
AS $$
DECLARE
    _quadkey TEXT;
    _mask INT;
    _digit INT;
BEGIN
    _quadkey := '';
    FOR i IN REVERSE level_of_detail..1 LOOP
        _digit := 0;
        _mask := 1 << (i - 1);

        IF (tile_x & _mask) != 0 THEN _digit := _digit + 1; 
        END IF;

        IF (tile_y & _mask) != 0 THEN _digit := _digit + 2;
        END IF;

        _quadkey = CONCAT(_quadkey, _digit);
    END LOOP;
    RETURN _quadkey;
END;
$$;
