-- Copyright (C) 2023:
--   Helmholtz-Zentrum Potsdam Deutsches GeoForschungsZentrum GFZ
--
-- This program is free software: you can redistribute it and/or modify it
-- under the terms of the GNU Affero General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
-- General Public License for more details.
--
-- You should have received a copy of the GNU Affero General Public License
-- along with this program. If not, see http://www.gnu.org/licenses/.

-- Creates a PostGIS geometry from a Quadkey value.
-- Adapted from https://learn.microsoft.com/en-us/bingmaps/articles/bing-maps-tile-system

CREATE OR REPLACE FUNCTION quadkey_to_geom(quadkey TEXT)
RETURNS GEOMETRY
LANGUAGE plpgsql IMMUTABLE PARALLEL SAFE
AS $$
DECLARE
    _t RECORD;
    _p_1 RECORD;
    _p_2 RECORD;
    _c_1 RECORD;
    _c_2 RECORD;
BEGIN
    _t := quadkey_to_tile_xy(quadkey);
    _p_1 := tile_xy_to_pixel_xy(_t.tile_x, _t.tile_y);
    _p_2 := tile_xy_to_pixel_xy(_t.tile_x + 1, _t.tile_y + 1);
    _c_1 := pixel_xy_to_latlon(_p_1.pixel_x, _p_1.pixel_y, LENGTH(quadkey));
    _c_2 := pixel_xy_to_latlon(_p_2.pixel_x, _p_2.pixel_y, LENGTH(quadkey));
    RETURN ST_MakeEnvelope(
        _c_1.longitude, _c_1.latitude, _c_2.longitude, _c_2.latitude, 4326);
END;
$$;
